"""
Autogenerated using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__


"""
from collections import OrderedDict
from typing import Any
from typing import Dict

from idem_azure_auto.helpers.exc import DescribeError
from idem_azure_auto.helpers.returns import StateReturn

__contracts__ = ["resource"]


async def present(
    hub,
    ctx,
    name: str,
    resource_group_name: str,
    virtual_network_name: str,
    subnet_name: str,
    parameters: dict = {},
    force_update: bool = False,
) -> StateReturn:
    r"""
    **Autogenerated function**

    Create Subnets

    Args:
        name(str): The identifier for this state.
        resource_group_name(str): The name of the resource group.
        virtual_network_name(str): The name of the virtual network.
        subnet_name(str): The name of the subnet.
        parameters(dict, optional): API request payload parameters. Defaults to {}.
        force_update(bool, optional): If PUT operation should be used instead of PATCH operation during resource update. Defaults to False.

    Returns:
        StateReturn

    Examples:

        .. code-block:: sls

            resource_is_present:
              azure.virtual_networks.subnets.present:
                - name: value
                - resource_group_name: value
                - virtual_network_name: value
                - subnet_name: value
    """

    subscription_id = ctx.acct.subscription_id
    response_get = await hub.exec.request.json.get(
        ctx,
        url=f"https://management.azure.com/subscriptions/{subscription_id}/resourceGroups/{resource_group_name}/providers/Microsoft.Network/virtualNetworks/{virtual_network_name}/subnets/{subnet_name}?api-version=2021-03-01",
        success_codes=[200],
        headers=ctx.acct.headers,
    )

    if force_update:
        if ctx.get("test", False):
            return StateReturn(
                name=name,
                result=True,
                comment="Would force to update azure.virtual_networks.subnets",
            )
        response_force_put = await hub.exec.request.json.put(
            ctx,
            url=f"https://management.azure.com/subscriptions/{subscription_id}/resourceGroups/{resource_group_name}/providers/Microsoft.Network/virtualNetworks/{virtual_network_name}/subnets/{subnet_name}?api-version=2021-03-01",
            success_codes=[200, 201],
            headers=ctx.acct.headers,
            json=parameters,
        )
        if response_force_put["result"]:
            old_resource = response_get["ret"] if response_get["result"] else None
            return StateReturn(
                name=name,
                result=True,
                old_obj=old_resource,
                new_obj=response_force_put["ret"],
                comment=response_force_put["comment"],
            )
        else:
            hub.log.debug(
                f"Could not force to update Subnets {response_force_put['comment']} {response_force_put['ret']}"
            )
            return StateReturn(
                name=name,
                result=False,
                comment=response_force_put["comment"],
                error=response_force_put["ret"],
            )

    if not response_get["result"]:
        if ctx.get("test", False):
            return StateReturn(
                name=name,
                result=True,
                comment="Would create azure.virtual_networks.subnets",
            )

        if response_get["status"] == 404:
            # PUT operation to create a resource
            response_put = await hub.exec.request.json.put(
                ctx,
                url=f"https://management.azure.com/subscriptions/{subscription_id}/resourceGroups/{resource_group_name}/providers/Microsoft.Network/virtualNetworks/{virtual_network_name}/subnets/{subnet_name}?api-version=2021-03-01",
                success_codes=[200, 201],
                headers=ctx.acct.headers,
                json=parameters,
            )

            if not response_put["result"]:
                hub.log.debug(
                    f"Could not create Subnets {response_put['comment']} {response_put['ret']}"
                )
                return StateReturn(
                    name=name,
                    result=False,
                    comment=response_put["comment"],
                    error=response_put["ret"],
                )

            return StateReturn(
                name=name,
                result=True,
                old_obj=None,
                new_obj=response_put["ret"],
                comment=response_put["comment"],
            )
        else:
            hub.log.debug(
                f"Could not get Subnets {response_get['comment']} {response_get['ret']}"
            )
            return StateReturn(
                name=name,
                result=False,
                comment=response_get["comment"],
                error=response_get["ret"],
            )

    # No update operation on Subnets since Azure does not have PATCH api on Subnets
    return StateReturn(
        name=name,
        result=True,
        old_obj=response_get["ret"],
        new_obj=response_get["ret"],
        comment=response_get["comment"],
    )


async def absent(
    hub,
    ctx,
    name: str,
    resource_group_name: str,
    virtual_network_name: str,
    subnet_name: str,
) -> StateReturn:
    r"""
    **Autogenerated function**

    Delete Subnets

    Args:
        name(str): The identifier for this state.
        resource_group_name(str): The name of the resource group.
        virtual_network_name(str): The name of the virtual network.
        subnet_name(str): The name of the subnet.

    Returns:
        StateReturn

    Examples:

        .. code-block:: sls

            resource_is_absent:
              azure.virtual_networks.subnets.absent:
                - name: value
                - resource_group_name: value
                - virtual_network_name: value
                - subnet_name: value
    """

    subscription_id = ctx.acct.subscription_id
    response_get = await hub.exec.request.json.get(
        ctx,
        url=f"https://management.azure.com/subscriptions/{subscription_id}/resourceGroups/{resource_group_name}/providers/Microsoft.Network/virtualNetworks/{virtual_network_name}/subnets/{subnet_name}?api-version=2021-03-01",
        success_codes=[200],
        headers=ctx.acct.headers,
    )
    if response_get["result"]:
        if ctx.get("test", False):
            return StateReturn(
                name=name,
                result=True,
                comment="Would delete azure.virtual_networks.subnets",
            )

        existing_resource = response_get["ret"]
        response_delete = await hub.exec.request.raw.delete(
            ctx,
            url=f"https://management.azure.com/subscriptions/{subscription_id}/resourceGroups/{resource_group_name}/providers/Microsoft.Network/virtualNetworks/{virtual_network_name}/subnets/{subnet_name}?api-version=2021-03-01",
            success_codes=[200, 202, 204],
            headers=ctx.acct.headers,
        )

        if not response_delete["result"]:
            hub.log.debug(
                f"Could not delete Subnets {response_delete['comment']} {response_delete['ret']}"
            )
            return StateReturn(
                name=name,
                result=False,
                comment=response_delete["comment"],
                error=response_delete["ret"],
            )

        return StateReturn(
            name=name,
            result=True,
            old_obj=existing_resource,
            new_obj={},
            comment=response_delete["comment"],
        )
    elif response_get["status"] == 404:
        # If Azure returns 'Not Found' error, it means the resource has been absent.
        return StateReturn(
            name=name,
            result=True,
            old_obj=None,
            new_obj=None,
            comment=f"'{name}' already absent",
        )
    else:
        hub.log.debug(
            f"Could not get Subnets {response_get['comment']} {response_get['ret']}"
        )
        return StateReturn(
            name=name,
            result=False,
            comment=response_get["comment"],
            error=response_get["ret"],
        )


async def describe(hub, ctx) -> Dict[str, Dict[str, Any]]:
    r"""
    **Autogenerated function**

    Describe the resource in a way that can be recreated/managed with the corresponding "present" function


    List all Subnets under the same subscription


    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: bash

            $ idem describe azure.virtual_networks.subnets
    """

    result = {}
    subscription_id = ctx.acct.subscription_id
    uri_parameters = OrderedDict(
        {
            "resourceGroups": "resource_group_name",
            "virtualNetworks": "virtual_network_name",
            "subnets": "subnet_name",
        }
    )
    try:
        async for vnet_page_result in hub.tool.azure.request.paginate(
            ctx,
            url=f"https://management.azure.com/subscriptions/{subscription_id}/providers/Microsoft.Network/virtualNetworks?api-version=2021-03-01",
            success_codes=[200],
            headers=ctx.acct.headers,
        ):
            vnet_list = vnet_page_result.get("value", None)
            if vnet_list:
                for vnet in vnet_list:
                    url = f"https://management.azure.com/{vnet.get('id')}/subnets?api-version=2021-03-01"
                    response_get = await hub.exec.request.json.get(
                        ctx,
                        url=url,
                        success_codes=[200],
                        headers=ctx.acct.headers,
                    )
                    if response_get["result"]:
                        resource_list = response_get["ret"].get("value", list())
                        for resource in resource_list:
                            uri_parameter_values = (
                                hub.tool.azure.utils.get_uri_parameter_value_from_uri(
                                    resource["id"], uri_parameters
                                )
                            )
                            result[resource["id"]] = {
                                f"azure.virtual_networks.subnets.present": uri_parameter_values
                                + [{"parameters": resource}]
                            }
                    else:
                        raise ValueError(
                            f"Error on requesting GET {url} with status code {response_get['status_code']}:"
                            f" {response_get.get('ret', '')}"
                        )
    except ValueError as e:
        raise DescribeError(f"Error on describing Subnets: {str(e)}")
    return result
