"""
Autogenerated using `pop-create-idem <https://gitlab.com/saltstack/pop/pop-create-idem>`__


"""
from collections import OrderedDict
from typing import Any
from typing import Dict

from idem_azure_auto.helpers.exc import DescribeError
from idem_azure_auto.helpers.returns import StateReturn


async def present(
    hub,
    ctx,
    name: str,
    disk_access_name: str,
    resource_group_name: str,
    parameters: dict = {},
) -> Dict[str, Any]:
    r"""
    **Autogenerated function**

    Create or update Disk Accesses

    Args:
        name(str): The identifier for this state.
        disk_access_name(str): The name of the disk access resource that is being created. The name can't be changed after the disk encryption set is created. Supported characters for the name are a-z, A-Z, 0-9 and _. The maximum name length is 80 characters.
        resource_group_name(str): The name of the resource group.
        parameters(dict, optional): API request payload parameters. Defaults to {}.

    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: sls

            resource_is_present:
              azure_auto.compute.disk_accesses.present:
                - name: value
                - disk_access_name: value
                - resource_group_name: value
    """

    subscription_id = ctx.acct.subscription_id
    response_get = await hub.exec.request.json.get(
        ctx,
        url=f"https://management.azure.com/subscriptions/{subscription_id}/resourceGroups/{resource_group_name}/providers/Microsoft.Compute/diskAccesses/{disk_access_name}?api-version=2020-12-01",
        success_codes=[200],
        headers=ctx.acct.headers,
    )

    if not response_get["status"]:
        if ctx.get("test", False):
            return StateReturn(
                name=name,
                result=True,
                comment="Would create azure.compute.disk_accesses",
            )

        if response_get["status_code"] == 404:
            # PUT operation to create a resource
            response_put = await hub.exec.request.json.put(
                ctx,
                url=f"https://management.azure.com/subscriptions/{subscription_id}/resourceGroups/{resource_group_name}/providers/Microsoft.Compute/diskAccesses/{disk_access_name}?api-version=2020-12-01",
                success_codes=[200, 201],
                headers=ctx.acct.headers,
                json=parameters,
            )

            if not response_put["status"]:
                hub.log.debug(
                    f"Could not create Disk Accesses {response_put['comment']} {response_put['ret']}"
                )
                return StateReturn(
                    name=name,
                    result=False,
                    comment=response_put["comment"],
                    error=response_put["ret"],
                )

            return StateReturn(
                name=name,
                result=True,
                old_obj=None,
                new_obj=response_put["ret"],
                comment=response_put["comment"],
            )
        else:
            hub.log.debug(
                f"Could not get Disk Accesses {response_get['comment']} {response_get['ret']}"
            )
            return StateReturn(
                name=name,
                result=False,
                comment=response_get["comment"],
                error=response_get["ret"],
            )

    else:
        # PATCH operation to update a resource
        patch_parameters = {"tags": "tags"}
        existing_resource = response_get["ret"]
        new_parameters = hub.tool.azure.utils.patch_json_content(
            patch_parameters, existing_resource, parameters
        )
        if ctx.get("test", False):
            return StateReturn(
                name=name,
                result=True,
                comment=f"Would update azure.compute.disk_accesses with parameters: {new_parameters}",
            )

        if not new_parameters:
            return StateReturn(
                name=name,
                result=True,
                old_obj=existing_resource,
                new_obj=existing_resource,
                comment=f"'{name}' has no property need to be updated.",
            )

        response_patch = await hub.exec.request.json.patch(
            ctx,
            url=f"https://management.azure.com/subscriptions/{subscription_id}/resourceGroups/{resource_group_name}/providers/Microsoft.Compute/diskAccesses/{disk_access_name}?api-version=2020-12-01",
            success_codes=[200],
            headers=ctx.acct.headers,
            json=new_parameters,
        )

        if not response_patch["status"]:
            hub.log.debug(
                f"Could not update Disk Accesses {response_patch['comment']} {response_patch['ret']}"
            )
            return StateReturn(
                name=name,
                result=False,
                comment=response_patch["comment"],
                error=response_patch["ret"],
            )

        return StateReturn(
            name=name,
            result=True,
            old_obj=existing_resource,
            new_obj=response_patch["ret"],
            comment=response_patch["comment"],
        )


async def absent(
    hub, ctx, name: str, disk_access_name: str, resource_group_name: str
) -> Dict[str, Any]:
    r"""
    **Autogenerated function**

    Delete Disk Accesses

    Args:
        name(str): The identifier for this state.
        disk_access_name(str): The name of the disk access resource that is being created. The name can't be changed after the disk encryption set is created. Supported characters for the name are a-z, A-Z, 0-9 and _. The maximum name length is 80 characters.
        resource_group_name(str): The name of the resource group.

    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: sls

            resource_is_absent:
              azure_auto.compute.disk_accesses.absent:
                - name: value
                - disk_access_name: value
                - resource_group_name: value
    """

    subscription_id = ctx.acct.subscription_id
    response_get = await hub.exec.request.json.get(
        ctx,
        url=f"https://management.azure.com/subscriptions/{subscription_id}/resourceGroups/{resource_group_name}/providers/Microsoft.Compute/diskAccesses/{disk_access_name}?api-version=2020-12-01",
        success_codes=[200],
        headers=ctx.acct.headers,
    )
    if response_get["status"]:
        if ctx.get("test", False):
            return StateReturn(
                name=name,
                result=True,
                comment="Would delete azure.compute.disk_accesses",
            )

        existing_resource = response_get["ret"]
        response_delete = await hub.exec.request.raw.delete(
            ctx,
            url=f"https://management.azure.com/subscriptions/{subscription_id}/resourceGroups/{resource_group_name}/providers/Microsoft.Compute/diskAccesses/{disk_access_name}?api-version=2020-12-01",
            success_codes=[200, 202, 204],
            headers=ctx.acct.headers,
        )

        if not response_delete["status"]:
            hub.log.debug(
                f"Could not delete Disk Accesses {response_delete['comment']} {response_delete['ret']}"
            )
            return StateReturn(
                name=name,
                result=False,
                comment=response_delete["comment"],
                error=response_delete["ret"],
            )

        return StateReturn(
            name=name,
            result=True,
            old_obj=existing_resource,
            new_obj={},
            comment=response_delete["comment"],
        )
    elif response_get["status_code"] == 404:
        # If Azure returns 'Not Found' error, it means the resource has been absent.
        return StateReturn(
            name=name,
            result=True,
            old_obj=None,
            new_obj=None,
            comment=f"'{name}' already absent",
        )
    else:
        hub.log.debug(
            f"Could not get Disk Accesses {response_get['comment']} {response_get['ret']}"
        )
        return StateReturn(
            name=name,
            result=False,
            comment=response_get["comment"],
            error=response_get["ret"],
        )


async def describe(hub, ctx) -> Dict[str, Dict[str, Any]]:
    r"""
    **Autogenerated function**

    Describe the resource in a way that can be recreated/managed with the corresponding "present" function


    List all Disk Accesses under the same subscription


    Returns:
        Dict[str, Any]

    Examples:

        .. code-block:: bash

            $ idem describe azure_auto.compute.disk_accesses
    """

    result = {}
    subscription_id = ctx.acct.subscription_id
    uri_parameters = OrderedDict(
        {"diskAccesses": "disk_access_name", "resourceGroups": "resource_group_name"}
    )
    try:
        async for page_result in hub.exec.azure.request.paginate(
            ctx,
            url=f"https://management.azure.com/subscriptions/{subscription_id}/providers/Microsoft.Compute/diskAccesses?api-version=2020-12-01",
            success_codes=[200],
            headers=ctx.acct.headers,
        ):
            resource_list = page_result.get("value", None)
            if resource_list:
                for resource in resource_list:
                    uri_parameter_values = (
                        hub.tool.azure.utils.get_uri_parameter_value_from_uri(
                            resource["id"], uri_parameters
                        )
                    )
                    result[resource["id"]] = {
                        f"azure.compute.disk_accesses.present": uri_parameter_values
                        + [{"parameters": resource}]
                    }
    except ValueError as e:
        raise DescribeError(f"Error on describing Disk Accesses: {str(e)}")
    return result
